package com.eigenpi.example1btstring;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

// cristinel ababei, 2017
// this project is just a bluetooth terminal that can be used to send a string and have
// echoed back from the arduino + hc-06; it is an adapted version of the project
// of omar aflak, described here:
// https://causeyourestuck.io/2015/12/14/communication-between-android-and-hc-06-module/
// i brought all libraries in here to make the project self-contained and easily understood;
// that is, the Bluetooth and PullToRefresh (which is a neat feature, but an unnecessary
// complication...); i made some changes in variable names to make it easier to read;
//
// TODO:
// --get rid of the pull to refresh; just have the button, which if pressed scans again;
// --when pressing the scan button, do not change layout to scan page; stay on select to reduce
// number of layout pages;
// --replace "cancel connection" from the menu bar with a button at the bottom pf page?
//
// this is the landing UI page initially;

public class Select extends Activity implements PullToRefresh.OnRefreshListener {
    // next two are custom classes to make it easier to handle a BT connection and
    // to implement a neat feature of refreshing list by pulling it;
    private Bluetooth bt_link; // see Bluetooth.java
    private PullToRefresh pull_to_refresh; // see PullToRefresh.java
    private Button not_found_button;
    private ListView list_of_paired_bt_devices;
    private List<BluetoothDevice> paired;
    private boolean registered=false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.select);

        IntentFilter filter = new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED);
        registerReceiver(mReceiver, filter);
        registered=true;

        bt_link = new Bluetooth(this);
        bt_link.enableBluetooth();

        pull_to_refresh = (PullToRefresh) findViewById(R.id.pull_to_refresh);
        list_of_paired_bt_devices = (ListView) findViewById(R.id.list);
        not_found_button = (Button) findViewById(R.id.not_in_list);

        pull_to_refresh.setListView(list_of_paired_bt_devices);
        pull_to_refresh.setOnRefreshListener(this);
        pull_to_refresh.setSlide(500);

        list_of_paired_bt_devices.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                Intent i = new Intent(Select.this, Chat.class);
                i.putExtra("pos", position);
                if(registered) {
                    unregisterReceiver(mReceiver);
                    registered=false;
                }
                startActivity(i);
                finish();
            }
        });

        not_found_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(Select.this, Scan.class);
                startActivity(i);
            }
        });

        // use bt to update list_of_paired_bt_devices that are displayed as available;
        addDevicesToList();
    }

    @Override
    public void onRefresh() {
        List<String> names = new ArrayList<String>();
        for (BluetoothDevice d : bt_link.getPairedDevices()){
            names.add(d.getName());
        }

        String[] array = names.toArray(new String[names.size()]);
        final ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, android.R.id.text1, array);

        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                list_of_paired_bt_devices.removeViews(0, list_of_paired_bt_devices.getCount());
                list_of_paired_bt_devices.setAdapter(adapter);
                paired = bt_link.getPairedDevices();
            }
        });
        pull_to_refresh.refreshComplete();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if(registered) {
            unregisterReceiver(mReceiver);
            registered=false;
        }
    }

    // this function adds BT devices found and paired to, to the list
    // of devices that are shown and from among which one can be selected
    // to connect to and to send strings to;
    private void addDevicesToList() {
        paired = bt_link.getPairedDevices();

        List<String> names = new ArrayList<>();
        for (BluetoothDevice d : paired){
            names.add(d.getName());
        }
        String[] array = names.toArray(new String[names.size()]);

        ArrayAdapter<String> adapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, android.R.id.text1, array);

        list_of_paired_bt_devices.setAdapter(adapter);
        not_found_button.setEnabled(true);
    }

    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            final String action = intent.getAction();

            if (action.equals(BluetoothAdapter.ACTION_STATE_CHANGED)) {
                final int state = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, BluetoothAdapter.ERROR);

                switch (state) {
                    case BluetoothAdapter.STATE_OFF:
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                list_of_paired_bt_devices.setEnabled(false);
                            }
                        });
                        Toast.makeText(Select.this, "Turn on bluetooth", Toast.LENGTH_LONG).show();
                        break;
                    case BluetoothAdapter.STATE_ON:
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                addDevicesToList();
                                list_of_paired_bt_devices.setEnabled(true);
                            }
                        });
                        break;
                }
            }
        }
    };
}

